package com.clx.performance.dao.impl.export;

import com.clx.performance.dao.export.ExportTemplateDao;
import com.clx.performance.mapper.export.ExportTemplateMapper;
import com.clx.performance.model.export.ExportTemplate;
import com.msl.common.base.Optional;
import com.msl.common.dao.impl.BaseDaoImpl;
import com.msl.common.enums.WhetherEnum;
import com.msl.user.utils.TokenUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import java.util.Collections;
import java.util.List;

@Repository
public class ExportTemplateDaoImpl extends BaseDaoImpl<ExportTemplateMapper, ExportTemplate, Integer> implements ExportTemplateDao {

    /**
     * 根据功能code集合查询某用户下的模板列表
     *
     * @param userNo       用户编码
     * @param functionCode 功能code
     * @return
     */
    @Override
    public List<ExportTemplate> listUserFunctionTemplate(Long userNo, String functionCode) {
        if (userNo == null || StringUtils.isBlank(functionCode)) {
            return Collections.emptyList();
        }
        return list(lQrWrapper().eq(ExportTemplate::getUserNo, userNo)
                .eq(ExportTemplate::getExportFunctionCode, functionCode)
                .orderByDesc(ExportTemplate :: getId));
    }

    /**
     * 根据名称查询用户在某功能下的模板
     *
     * @param userNo       用户编码
     * @param functionCode 功能code
     * @param name         模板名称
     * @return
     */
    @Override
    public Optional<ExportTemplate> getUserFunctionTemplateByName(Long userNo, String functionCode, String name) {
        return Optional.ofNullable(userNo)
                .filter(no -> StringUtils.isNoneBlank(functionCode, name))
                .map(no -> lQrWrapper().eq(ExportTemplate::getUserNo, no)
                        .eq(ExportTemplate::getExportFunctionCode, functionCode)
                        .eq(ExportTemplate::getName, name))
                .map(super::getOne);

    }

    /**
     * 根据功能code集合查询某用户下的模板列表
     *
     * @param userNo        用户编码
     * @param functionCodes 功能code集合
     * @return
     */
    @Override
    public List<ExportTemplate> listUserFunctionsTemplate(Long userNo, List<String> functionCodes) {
        if (userNo == null || CollectionUtils.isEmpty(functionCodes)) {
            return Collections.emptyList();
        }
        return list(lQrWrapper().eq(ExportTemplate::getUserNo, userNo)
                .in(ExportTemplate::getExportFunctionCode, functionCodes));
    }

    /**
     * 根据id查询模板
     *
     * @param id 模板id
     * @return
     */
    @Override
    public Optional<ExportTemplate> getById(Integer id) {
        return Optional.ofNullable(id).map(super::getById);
    }

    /**
     * 新增模板
     *
     * @param template 待新增模板数据
     * @return 模板id
     */
    @Override
    public void saveTemplate(ExportTemplate template) {
        save(template);
    }

    /**
     * 删除模板
     *
     * @param id 模板id
     * @return 模板id
     */
    @Override
    public void deleteTemplate(Integer id) {
        Optional.ofNullable(id).ifPresent(super::removeById);
    }

    /**
     * 修改模板名称
     *
     * @param id   模板id
     * @param name 模板名称
     * @return 模板id
     */
    @Override
    public void updateTemplateName(Integer id, String name) {
        update(lUdWrapper().set(ExportTemplate::getName, name)
                .eq(ExportTemplate::getId, id));
    }

    /**
     * 去除用户在某个功能下的默认模板
     *
     * @param userNo       用户编码
     * @param functionCode 功能code
     */
    @Override
    public void updateTemplateNotDefault(Long userNo, String functionCode) {
        update(lUdWrapper().set(ExportTemplate::getDefaultFlag, WhetherEnum.get(false))
                .eq(ExportTemplate::getUserNo, userNo)
                .eq(ExportTemplate::getExportFunctionCode, functionCode)
                .eq(ExportTemplate::getCompanyNo, TokenUtil.getLoginUserInfo().getCompanyNo()));
    }

    /**
     * 设置模板为默认模板
     *
     * @param templateId 默认模板
     */
    @Override
    public void updateTemplateDefault(Integer templateId) {
        update(lUdWrapper().set(ExportTemplate::getDefaultFlag, WhetherEnum.get(true))
                .eq(ExportTemplate::getId, templateId));
    }

    /**
     * 统计用户在某个功能下的模板数量
     *
     * @param userNo       用户编码
     * @param functionCode 功能code
     */
    @Override
    public long countUserFunctionTemplate(Long userNo, String functionCode) {
        return Optional.ofNullable(userNo)
                .filter(no -> StringUtils.isNotBlank(functionCode))
                .map(no -> lQrWrapper().eq(ExportTemplate::getUserNo, no)
                        .eq(ExportTemplate::getExportFunctionCode, functionCode))
                .map(baseMapper::selectCount)
                .orElse(0L);
    }
}
